"""
Class for dealing with word embeddings
"""

import numpy as np

from keras.preprocessing.text import Tokenizer
from keras.preprocessing.sequence import pad_sequences

def load_from_text(path):
    """
    A functions that loads embeddings from a txt document
    """
    def get_coefs(word,*arr): return word, np.asarray(arr, dtype='float32')
    emb = dict(get_coefs(*o.split(" ")) for o in open(path, 
                                                      encoding = 'utf-8',
                                                      errors='ignore'))
    return emb

def tokenize_text(string_list, max_features, max_len):
    """
    Tokenizes a given string list
    max_featues - number of unique tokens to use in final matrix
    max_len - number of unique tokens to use in each list element
    """
    tokenizer = Tokenizer(num_words = max_features)
    tokenizer.fit_on_texts(string_list)
    token = tokenizer.texts_to_sequences(string_list)
    token = pad_sequences(token, maxlen = max_len)
    
    return token, tokenizer

def create_embedding_matrix(embeddings, tokenizer, max_features, 
                            embed_size = 300): 
    """
    Function to create the embedding matrix to use in neural networks
    embeddings - output of load_from_text() function
    tokenizer - output of tokenize_text() function
    max_features - constant; how many unique tokens to use
    embed_size - constant; how many coordinates does the read embedding has
    """
    embedding_matrix = np.zeros((max_features, embed_size))
    for word, index in tokenizer.word_index.items():
        if index > max_features - 1:
            break
        else:
            try:
                embedding_matrix[index] = embeddings[word]
            except:
                continue
    return embedding_matrix        
    


"""
Class for data download
"""

import pandas as pd
from sqlalchemy import create_engine

class Get_sql:
    
    def get_google_tree(connection):
        """
        An sql query to download the whole google taxonomy tree 
        """
        sql_query = "SELECT distinct(full_title), id, parent_id, title as class \
                        FROM sixads.sixads_category"

        tree = pd.read_sql_query(sql_query, connection)
        def f(x):
            if x is None:
                return ""
            else:
                return '>' + x
        tree['full_title'] = [f(x) for x in tree['full_title']]
        return tree

    def get_data(connection, select_part, from_part, where_part = ''):
        "Function that construcs a query from the given parts and executes it"
        
        columns = ",".join(select_part)
        
        sql_query = "SELECT {columns} \
                    FROM {from_part} \
                    {where_part}".format(columns = columns, 
                                         from_part = from_part, 
                                         where_part = where_part)
                    
        return pd.read_sql_query(sql_query, connection)

class Write_sql:
        
    def write_to_table(specs, table, data, if_exists = 'replace'):
        """
        Writes data to the desired table 
        """
        engine = create_engine("mysql+pymysql://{user}:{pw}@{host}/{db}"
                       .format(user=specs['user'],
                               pw=specs['password'],
                               host = specs['host'],
                               db=specs['db']))
        data.to_sql(table, con = engine, 
                            if_exists = if_exists, index = False)    


"""
Methods to work with images
"""

import cv2
import numpy as np
import urllib, requests
from io import BytesIO
from PIL import Image

def img_read_url(url, h = 256, w = 256, to_grey = False, timeout = 2):
    """
    Reads and preproces an image in *url* 
    
    h (float): height of the resized image
    w (float): width of the resized image.
    to_grey (bool): should the image be grey scale?
    """        
        
    resp = urllib.request.urlopen(url, timeout = timeout)
    image = np.asarray(bytearray(resp.read()), dtype="uint8")
    image = cv2.imdecode(image, cv2.IMREAD_COLOR)[...,::-1]
    image = cv2.resize(image, (h, w))
    if(to_grey):
        image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    return image   

def img_read_url_PIL(url, h = 256, w = 256,  timeout = 2):
    """
    Reads and preproces an image in *url* using the PIL framework
    
    h (float): height of the resized image
    w (float): width of the resized image.
    """
    
    response = requests.get(url)    
    image = Image.open(BytesIO(response.content))
    image = image.resize((h, w), Image.ANTIALIAS)
        
    return image   

def img_read(path, h = 256, w = 256, to_grey = False):
    """
    Reads and preproces an image in *path* 
    
    h (float): height of the resized image
    w (float): width of the resized image.
    to_grey (bool): should the image be grey scale?
    """
    image = cv2.imread(path)[...,::-1]
    image = cv2.resize(image, (h, w))
    if(to_grey):
        image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    return image 

def return_image_hist(image, no_bins_per_channel = 10):
    """ 
    Returns a vector representing the distribution of colors of an image
    """
    hist_container = []
    for channel in range(3):
        hist = cv2.calcHist([image], [channel], None, [no_bins_per_channel], [0, 256])    
        hist = [x[0] for x in hist.tolist()]
        hist_container += hist
    
    return hist_container    
